// DxDevice.cpp
//

#include "PursuitRender.h"

LPDIRECT3D9 PR_Direct3D;
LPDIRECT3DDEVICE9 PR_D3D_Device;
LPD3DXSPRITE PR_DxSprite;

PRENDER_API PDRESULT PRInitDxDevice(HWND hWnd, PCSettings settings)
{
	//D3DDISPLAYMODE ddm;

	D3DDISPLAYMODE adaptermode;
	PR_Direct3D = Direct3DCreate9(D3D_SDK_VERSION);
	
	PRGetAdapterMode(settings, &adaptermode);

	D3DPRESENT_PARAMETERS pp;
	memset(&pp, 0, sizeof(D3DPRESENT_PARAMETERS));

	pp.Windowed = !settings.fullscreen;
	pp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	pp.Flags = settings.deviceflags;
	pp.AutoDepthStencilFormat = D3DFMT_D16;
	pp.EnableAutoDepthStencil = true;
	pp.hDeviceWindow = hWnd;

	// Displaymode
	pp.BackBufferCount = 1;
	pp.BackBufferWidth = adaptermode.Width;
	pp.BackBufferHeight = adaptermode.Height;
	pp.BackBufferFormat = adaptermode.Format;
	if (settings.fullscreen) pp.FullScreen_RefreshRateInHz = adaptermode.RefreshRate;
	//pp.PresentationInterval = adaptermode.RefreshRate;
	
	// Multisampling
	pp.MultiSampleType = settings.multisampletype;
	pp.MultiSampleQuality = settings.multisamplequality;

	RECT winrect;
	winrect.right = adaptermode.Width;
	winrect.bottom = adaptermode.Height;

	int y = GetSystemMetrics(SM_CYCAPTION) + adaptermode.Height + GetSystemMetrics(SM_CYFRAME) * 2;
	int x = GetSystemMetrics(SM_CXFRAME) * 2 + adaptermode.Width;

	SetWindowPos(hWnd, NULL, CW_USEDEFAULT, CW_USEDEFAULT, x, y, SWP_NOMOVE | SWP_NOZORDER);	

	HRESULT hr = PR_Direct3D->CreateDevice(settings.adapterno, settings.devicetype, hWnd, settings.deviceflags, &pp, &PR_D3D_Device);

	if (FAILED(hr))
	{
		PCErrorDX(hr, "DirectX konnte nicht initialisiert werden!");
		return PDERROR_DIRECTXINIT;
	}

	PDRESULT res = PRInitDxSprites();
	
	if (PDFAILED(res))
		return res;

	res = PRInitText();
	
	if (PDFAILED(res))
		return res;

	return PD_OK;
}

PRENDER_API PDRESULT PRInitDxSprites(void)
{
	HRESULT hr = D3DXCreateSprite(PR_D3D_Device, &PR_DxSprite);

	if (FAILED(hr))
	{
		PCErrorDX(hr, "D3DX-Sprite konnte nicht erstellt werden!");
		
		return PDERROR_DXSPRITEINIT;
	}

	return PD_OK;
}

PRENDER_API void PRGetAdapterMode(PCSettings settings, D3DDISPLAYMODE* adaptermode)
{
	LPDIRECT3D9 d3dtemp = Direct3DCreate9(D3D_SDK_VERSION);
	D3DDISPLAYMODE ddm;
	
	d3dtemp->GetAdapterDisplayMode(settings.adapterno, &ddm);

	// Unntig
	//int modecount = PR_Direct3D->GetAdapterModeCount(settings.adapterno, ddm.Format);

	d3dtemp->EnumAdapterModes(settings.adapterno, ddm.Format, settings.modeno, adaptermode);

	d3dtemp->Release();
}

PRENDER_API void PRPresent(void)
{
	if (PR_D3D_Device != NULL)
		PR_D3D_Device->Present(NULL, NULL, NULL, NULL);
}

PRENDER_API void PRRelease(void)
{
	if (PR_DxSprite != NULL) PR_DxSprite->Release();
	if (PR_Direct3D != NULL) PR_Direct3D->Release();
	if (PR_D3D_Device != NULL) PR_D3D_Device->Release();
}