using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using PursuitDrift.GameObjects;
using PursuitDrift.GameObjects.PhysicObjects;
using PursuitDrift.GameObjects.PhysicObjects.Cars;
using PursuitDrift.Interfaces;
using PursuitDrift.Interfaces.GameObjects;
using PursuitDrift.Physics;
using TomShane.Neoforce.Controls;

using Button = TomShane.Neoforce.Controls.Button;
using ButtonState = Microsoft.Xna.Framework.Input.ButtonState;
using Keys = Microsoft.Xna.Framework.Input.Keys;

namespace PursuitDrift
{
    /// <summary>
    /// This is the main type for PursuitDrift
    /// </summary>
    public class PursuitDriftGame : Game, IPursuitDriftGame
    {
        private readonly GraphicsDeviceManager _graphics;
        private SpriteBatch _spriteBatch;

        private SpriteFont _gameFont;
        private SpriteFont _debugFont;

        private KeyboardState _prevKeyState;

        private Manager _controlsManager;

        #region Constants
        public const string AudioDirectoryName = "Audio";
        public const string FontsDirectoryName = "Fonts";
        public const string ModelsDirectoryName = "Models";
        public const string ShadersDirectoryName = "Shaders";
        public const string TexturesDirectoryName = "Textures";
        #endregion

        #region GameObjects
        private Camera _camera;
        private DebugWriter _debugWriter;

        // Wird durch neoforce ersetzt.
        //private CursorObject _cursorObject;

        private CarObject _testCar;
        #endregion

        public PursuitDriftGame()
        {
            _graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            IsMouseVisible = true;

            CreateComponents();
        }

        #region Properties
        /// <summary>
        /// Gibt die SpriteBatch des Spiels zurck.
        /// </summary>
        public SpriteBatch SpriteBatch
        {
            get { return _spriteBatch; }
        }

        /// <summary>
        /// Gibt den GraphicsDeviceManager des Spieles zurck.
        /// </summary>
        public GraphicsDeviceManager DeviceManager
        {
            get { return _graphics; }
        }

        /// <summary>
        /// Gibt die InGame-Font zurck.
        /// </summary>
        public SpriteFont GameFont
        {
            get { return _gameFont; }
        }

        /// <summary>
        /// Gibt die Debug-Font zurck.
        /// </summary>
        public SpriteFont DebugFont
        {
            get { return _debugFont; }
        }

        /// <summary>
        /// Gibt die Kamera des Spieles zurck.
        /// </summary>
        public ICamera Camera
        {
            get { return _camera; }
        }
        #endregion

        #region Methods

        private void CreateComponents()
        {
            _camera = new Camera(this, this);
            Components.Add(_camera);

            CarInformation carinfo = CarInformation.Deserialize("Cars\\XnaCar.pdcar");

            _testCar = new CarObject(this, this, carinfo);
            Components.Add(_testCar);

            _debugWriter = new DebugWriter(this, this);
            Components.Add(_debugWriter);

            //_cursorObject = new CursorObject(this)
            //                    {
            //                        DrawOrder = 1
            //                    };
            //Components.Add(_cursorObject);

            //XnaButton _btn = new XnaButton(this, "Ein Button", Vector2.One * 50);
            //Components.Add(_btn);

            _controlsManager = new Manager(this, _graphics, "Default", true);
            _controlsManager.SkinDirectory = ".\\Skins\\";
            _controlsManager.Cursor = Cursors.Default;
            
        }

        private void CreateControls()
        {
            // Controls always have to be created and initialized after Manager.Initialize() is called.            

            
            // Create Button control and set the previous window as its parent.
            Button button = new Button(_controlsManager);
            button.Init();
            button.Text = "Neues Fenster";
            button.Width = 100;
            button.Height = 24;
            button.Left = 100;
            button.Top = 10;
            button.Click += (s, e) =>
                                {
                                    Window window = new Window(_controlsManager);
                                    window.Init();
                                    window.Text = "Getting Started";
                                    window.Width = 480;
                                    window.Height = 200;
                                    window.Center();
                                    window.Visible = true;
                                    _controlsManager.Add(window);
                                };
            //button.Parent = window;

            _controlsManager.Add(button);

            Button resetbtn = new Button(_controlsManager);
            resetbtn.Init();
            resetbtn.Text = "Auto zurcksetzten";
            resetbtn.Width = 120;
            resetbtn.Height = 24;
            resetbtn.Left = 220;
            resetbtn.Top = 10;
            resetbtn.Click += (s, e) => _testCar.Reset();

            // Add the window control to the manager processing queue.
            _controlsManager.Add(resetbtn);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            PhysX.InitializePhysX(this);

            _prevKeyState = new KeyboardState();

            GraphicsDevice.RenderState.CullMode = CullMode.None;
            GraphicsDevice.RenderState.AlphaBlendEnable = true;

            base.Initialize();

            // Noch brauchen wir keine
            CreateControls();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            _spriteBatch = new SpriteBatch(GraphicsDevice);

            _gameFont = Content.Load<SpriteFont>(FontsDirectoryName + "\\Game");
            _debugFont = Content.Load<SpriteFont>(FontsDirectoryName + "\\Debug");

#if DEBUG
            _debugWriter.Enabled = true;
            PhysX.IsDebugViewEnabled = true;
#else
            _debugWriter.Enabled = false;
            PhysX.IsDebugViewEnabled = false;
#endif

            base.LoadContent();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (Keyboard.GetState().IsKeyDown(Keys.Escape) || GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                Exit();

            _testCar.Turn(0.0f);

            KeyboardState _keyState = Keyboard.GetState();
            
            if (_keyState.IsKeyDown(Keys.F11) && _prevKeyState.IsKeyUp(Keys.F11))
            {
                _debugWriter.ToggleDebugView();
                PhysX.ToggleDebugView();
            }
            if (_keyState.IsKeyDown(Keys.Up))
            {
                _testCar.Accelerate(10f);
            }
            else if (_keyState.IsKeyDown(Keys.Down))
            {
                _testCar.Accelerate(-10f);
            }
            if (_keyState.IsKeyDown(Keys.Left))
            {
                _testCar.Turn(-0.5f);
            }
            else if (_keyState.IsKeyDown(Keys.Right))
            {
                _testCar.Turn(0.5f);
            }

            PhysX.UpdatePhysX(gameTime);

            _prevKeyState = _keyState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            PhysX.DrawVisualiation(this, this);

            base.Draw(gameTime);
        }
        #endregion
    }
}
